// The MIT License (MIT)
//
// Copyright (c) 2020 nslogx
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.

import 'package:arcana_auth_flutter/common.dart';
import 'package:flutter/material.dart';
import 'package:webview_flutter/webview_flutter.dart';

class ArcanaSdk {
  /// contorller of webview.
  WebViewController? controller;

  /// overlay.
  late OverlayEntry _overlayEntry;

  /// overlay state.
  late OverlayState _overlayState;

  ///visibility of webview
  bool _isVisible = false;

  ///fullscreen of the webview
  bool fullScreen = false;

  ///height of the webview
  double height = 200;

  ///width of the webview
  double width = 200;

  ///alignment of the webview
  Alignment alignment = Alignment.center;

  ///context of the webview
  late BuildContext context;

  ///login success callback
  late Function onLoginSuccess;

  ///log callback
  late Function onLog;

  ///user info callback
  Function? onUserInfoResponse;

  ///transaction callback
  Function? onTransactionResponse;

  ///account callback
  Function? getAccountResponse;

  ///logout callback
  Function? onLogoutResponse;

  ///onSendRequest callback
  Function? onSendRequestResponse;

  ///initialization of the webview
  initWeb(
      {required bool fullScreen,
      required double height,
      required double width,
      required Alignment alignment,
      required BuildContext context,
      required Function() onLoginSuccess,
      required Function() onLoginFailed,
      required Function(String error) onError,
      required ArcanaConfig config,
      required bool isPasswordLessLogin,
      required String clientId,
      String? email,
      required Function(String value) onLog}) {
    this.context = context;
    this.fullScreen = fullScreen;
    this.height = height;
    this.width = width;
    this.alignment = alignment;
    this.onLoginSuccess = onLoginSuccess;
    this.onLog = onLog;
    controller = WebViewController()
      ..setJavaScriptMode(JavaScriptMode.unrestricted)
      ..setBackgroundColor(const Color(0x00000000))
      ..setUserAgent(
          'Mozilla/5.0 (Linux; Android 5.1.1) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/91.0.4472.164 Mobile Safari/535.19')
      ..addJavaScriptChannel('xarFlutter', onMessageReceived: (message) {
        this.onLog(message.message);
        if (message.message.contains('show')) {
          showArcanaView();
        } else if (message.message.contains('hide')) {
          hideArcanaView();
        } else if (message.message.contains('login_complete')) {
          onLoginSuccess();
        } else if (message.message.contains('get_user_info')) {
          if (onUserInfoResponse != null) {
            onUserInfoResponse!(
                message.message.replaceAll("get_user_info-", ""));
          }
        } else if (message.message.contains('sendTransaction')) {
          if (onTransactionResponse != null) {
            onTransactionResponse!(
                message.message.replaceAll("sendTransaction-", ""));
          }
        } else if (message.message.contains('sendRequest')) {
          if (onSendRequestResponse != null) {
            onSendRequestResponse!(
                message.message.replaceAll("sendRequest-", ""));
          }
        } else if (message.message.contains('getAccount')) {
          if (getAccountResponse != null) {
            getAccountResponse!(message.message.replaceAll("getAccount-", ""));
          }
        } else if (message.message.contains('logout')) {
          if (onLogoutResponse != null) {
            // destroyView();
            onLogoutResponse!("Logout Successfully");
          }
        }
      })
      ..setNavigationDelegate(
        NavigationDelegate(
          onProgress: (int progress) {},
          onPageStarted: (String url) {},
          onPageFinished: (String url) {},
          onWebResourceError: (WebResourceError error) {},
        ),
      )
      ..loadRequest(Uri.parse(
          getUrl(isPasswordLessLogin, clientId, email, config, onError)));
  }

  ///clear caches of the webview in arcana
  clearCache() {
    controller?.clearCache();
    var webViewCookie = WebViewCookieManager();
    webViewCookie.clearCookies();
    controller?.reload();
    destroyView();
  }

  ///create overlay of the webview in arcana
  OverlayEntry _createOverlayEntry() {
    return OverlayEntry(
      builder: (context) => Align(
          alignment: alignment,
          child: SizedBox(
            height: fullScreen ? null : height,
            width: fullScreen ? null : width,
            child: Scaffold(
              appBar: fullScreen
                  ? AppBar(
                      leading: CloseButton(
                          onPressed: () {
                            hideArcanaView();
                          },
                          color: Colors.black),
                    )
                  : null,
              body: Container(
                child: controller != null
                    ? WebViewWidget(
                        controller: controller ?? WebViewController())
                    : Container(),
              ),
            ),
          )),
    );
  }

  ///show view of the webview in arcana
  showArcanaView() async {
    if (controller != null) {
      if (!_isVisible) {
        _overlayState = Overlay.of(context);
        _overlayEntry = _createOverlayEntry();
        _overlayState.insert(_overlayEntry);
        _isVisible = true;
      }
    }
  }

  ///resize of the webview in arcana
  void resize(bool fullScreen, double height, double width) {
    this.fullScreen = fullScreen;
    this.height = height;
    this.width = width;
    hideArcanaView();
    showArcanaView();
  }

  ///hide of the webview in arcana
  void hideArcanaView() {
    _isVisible = false;
    _overlayEntry.remove();
  }

  ///clear caches of the webview in arcana
  getWebViewVisibleStatus() {
    return _isVisible;
  }

  ///remove view of the webview in arcana
  void destroyView() {
    _isVisible = false;
    _overlayEntry.remove();
  }

  ///injection of the webview in arcana
  void injectJavaScript(String data) {
    controller?.runJavaScript(
        '(function() {window.dispatchEvent(new MessageEvent("message", {data: $data}));})()');
  }

  ///user defined of the webview in arcana
  void userDefinedRequest(String data) {
    injectJavaScript(data);
  }

  ///send request of the webview in arcana
  void sendRequest(id,
      {required Function(String response) onSendRequestResponse}) {
    this.onSendRequestResponse = onSendRequestResponse;
    injectJavaScript('{ id: "sendRequest-$id", method: "" , params: [] }');
  }

  ///get user info of the webview in arcana
  void getUserInfo(id,
      {required Function(String response) onUserInfoResponse}) {
    this.onUserInfoResponse = onUserInfoResponse;
    injectJavaScript(
        '{type: "request", data: { id: "get_user_info-$id",type:"newData", method: "get_user_info", params: [] }}');
  }

  ///send transaction of the webview in arcana
  void sendTransaction(id, from, to, value,
      {required Function(String response) onTransactionResponse}) {
    this.onTransactionResponse = onTransactionResponse;
    injectJavaScript(
        '{type: "request",data: {id:"sendTransaction-$id",method: "eth_sendTransaction",params: [{from:$from,to:$to,value:$value},],},}');
  }

  ///get account of the webview in arcana
  void getAccount(id, {required Function(String response) getAccountResponse}) {
    this.getAccountResponse = getAccountResponse;
    injectJavaScript(
        '{type: "request",data: { id:"getAccount-$id", method: "eth_accounts", params: [] },}');
  }

  ///logout of the webview in arcana
  void logOut({required Function(String response) onLogoutResponse}) {
    this.onLogoutResponse = onLogoutResponse;
    injectJavaScript('{type: "logout",id:"logout"}');
  }

  ///combined url of the webview in arcana
  String getUrl(bool isPasswordLessLogin, String clientId, String? email,
      ArcanaConfig config, Function onError) {
    if (isPasswordLessLogin) {
      if (email != null && email.length != 0) {
        return "${getConfig(config)}/start?clientId=$clientId&loginSrc=flutter&type=passwordless&email=$email&theme=dark";
      } else {
        onError("Invalid Email");
        return "";
      }
    } else {
      return "${getConfig(config)}/start?clientId=$clientId&loginSrc=flutter&type=google&theme=dark";
    }
  }
}
